import java.io.*;
import java.util.*;

public class CellPhoneUI {
    private CellPhone _cp;

    private CellPhoneUI(CellPhone cp) {
	_cp = cp;
    }

    private void go() {
	System.out.println("Phone number: " + _cp.getPhoneNumber());

	PhoneBook pb = new PhoneBook(_cp);
	PhoneType lastType = PhoneType.MAIN;
	BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
	while (true) {
	    System.out.print("cp> ");
	    String cmd;
	    try {
		cmd = in.readLine();
	    } catch (Exception e) {
		System.out.println("Error reading command, exiting:");
		e.printStackTrace();
		return;
	    }

	    if (cmd == null) {
		cmd = "quit";
		System.out.println("");
	    }
	    cmd = cmd.toLowerCase();

	    try {
		if (cmd.startsWith("l")) {
		    for (PhoneBookEntry pbe: pb.getEntries())
			System.out.printf("%03d %-30s %-20s %s\n",
			    pbe.index(), pbe.name(), pbe.phoneNumber(),
			    pbe.type().name());
		} else if (cmd.startsWith("r")) {
		    System.out.print("Index to remove: ");
		    int idx = Integer.parseInt(in.readLine());
		    PhoneBookEntry pbe = pb.findByIndex(idx);
		    if (pbe == null)
			throw new RuntimeException("No such index: " + idx);
		    System.out.print("Remove " + pbe.name() + " (Y/N)? ");
		    String confirm = in.readLine();
		    if (confirm.toLowerCase().startsWith("y")) {
			pb.removeEntry(pbe);
			System.out.println("Removed entry " + idx);
		    } else {
			System.out.println("Not removing entry " + idx);
		    }
		} else if (cmd.startsWith("a")) {
		    System.out.print("Name: ");
		    String name = in.readLine();
		    System.out.print("Number: ");
		    String phoneNumber = in.readLine();
		    System.out.print("Type [" + lastType.name() + "]: ");
		    String typeStr = in.readLine();
		    if (typeStr.equals(""))
			typeStr = lastType.name();
		    PhoneType type = PhoneType.findByName(typeStr);
		    if (type == null)
			throw new RuntimeException("Unknown number type " + typeStr);
		    lastType = type;

		    PhoneBookEntry pbe = new PhoneBookEntry(1, phoneNumber, name, type);
		    pb.addEntry(pbe);
		    System.out.println("Entry added as number " + pbe.index() + ".");
		} else if (cmd.startsWith("q")) {
		    System.out.println("Exiting.");
		    return;
		} else if (cmd.startsWith("e")) {
		    System.out.print("Index of entry to edit: ");
		    int idx = Integer.parseInt(in.readLine());
		    PhoneBookEntry pbe = pb.findByIndex(idx);
		    if (pbe == null)
			throw new RuntimeException("No such entry: " + idx);

		    System.out.print("Name [" + pbe.name() + "]: ");
		    String name = in.readLine();
		    if (!name.equals(""))
			pbe.setName(name);

		    System.out.print("Phone number [" + pbe.phoneNumber() + "]: ");
		    String num = in.readLine();
		    if (!num.equals(""))
			pbe.setPhoneNumber(num);

		    System.out.print("Type [" + pbe.type().name() + "]: ");
		    String type = in.readLine();
		    if (!type.equals(""))
			pbe.setType(PhoneType.findByName(type));

		    pb.updateEntry(pbe);
		    System.out.println("Entry " + pbe.index() + " updated.");
		} else {
		    System.out.println("Unknown command " + cmd);
		    System.out.println("Valid commands: list, remove, add, edit, quit");
		}
	    } catch (Exception e) {
		System.out.println("Error during command " + cmd + ":");
		e.printStackTrace();
	    }
	}
    }

    public static void main(String[] args) {
	if (args.length != 1) {
	    System.out.println("Usage: CellPhoneUI device-name");
	    System.exit(-1);
	}

	File device = new File(args[0]);
	V60 cp = null;
	try {
	    cp = new V60(device);
	} catch (Exception e) {
	    e.printStackTrace();
	    System.exit(0);
	}

	CellPhoneUI ui = new CellPhoneUI(cp);

	try {
	    ui.go();
	} catch (Exception e) {
	    e.printStackTrace();
	}

	cp.close();
	System.exit(0);
    }
}
