import java.util.*;

public class PhoneBook {
    private CellPhone _cp;
    private Collection<PhoneBookEntry> _ents;

    private static class AlphaSort implements Comparator<PhoneBookEntry> {
	public int compare(PhoneBookEntry pbe1, PhoneBookEntry pbe2) {
	    int nameDiff = pbe1.name().compareTo(pbe2.name());
	    int phoneDiff = pbe1.phoneNumber().compareTo(pbe2.phoneNumber());
	    int typeDiff = pbe1.type().index() - pbe2.type().index();
	    if (nameDiff != 0)
		return nameDiff;
	    if (phoneDiff != 0)
		return phoneDiff;
	    if (typeDiff != 0)
		return typeDiff;
	    return pbe1.index() - pbe2.index();
	}
    }

    public PhoneBook(CellPhone cp) {
	_cp = cp;
	Collection<PhoneBookEntry> ents = cp.getPhoneBookEntries();
	_ents = new TreeSet<PhoneBookEntry>(new AlphaSort());
	_ents.addAll(ents);
    }

    public Collection<PhoneBookEntry> getEntries() {
	return Collections.unmodifiableCollection(_ents);
    }

    public PhoneBookEntry findByIndex(int idx) {
	for (PhoneBookEntry e: _ents)
	    if (e.index() == idx)
		return e;
	return null;
    }

    public void removeEntry(PhoneBookEntry pbe) {
	_cp.removePhoneBookEntry(pbe);
	_ents.remove(findByIndex(pbe.index()));
    }

    public void addEntry(PhoneBookEntry pbe) {
	int tryIndex = 1;
	while (findByIndex(pbe.index()) != null)
	    pbe.setIndex(tryIndex++);
	_cp.addPhoneBookEntry(pbe);
	_ents.add(pbe);
    }

    public void updateEntry(PhoneBookEntry pbe) {
	PhoneBookEntry oldEnt = findByIndex(pbe.index());
	if (oldEnt == null)
	    throw new RuntimeException("Updating a non-existant entry " + pbe.index());

	_cp.updatePhoneBookEntry(pbe);
	_ents.remove(oldEnt);
	_ents.add(pbe);
    }
}
