import java.io.*;
import javax.comm.*;
import java.util.regex.*;

public class SerialPort {
    public static boolean debug = false;

    private File _device;
    private CommPort _port;

    private PrintWriter _out;
    private BufferedReader _in;

    private static final int TIMEOUT = 1000;

    public SerialPort(File device) {
	_device = device;

	try {
	    CommPortIdentifier cpi =
		CommPortIdentifier.getPortIdentifier(device.getPath());
	    _port = cpi.open("SerialPort", 5000);
	} catch (NoSuchPortException e) {
	    throw new RuntimeException("Could not find port " + device, e);
	} catch (PortInUseException e) {
	    throw new RuntimeException("Port " + device + " is in use", e);
	}

	try {
	    _port.enableReceiveTimeout(TIMEOUT);
	} catch (Exception e) {}
	if (!_port.isReceiveTimeoutEnabled())
	    System.out.println("Receive timeouts not supported by device");

	try {
	    OutputStream os = _port.getOutputStream();
	    _out = new PrintWriter(os);

	    InputStream is = _port.getInputStream();
	    InputStreamReader isr = new InputStreamReader(is);
	    _in = new BufferedReader(isr);
	} catch (IOException e) {
	    throw new RuntimeException(e);
	}
    }

    public void println(String cmd) {
	String send = cmd + "\r";
	_out.print(send);
	_out.flush();

	if (debug)
	    System.out.println("SP>> " + cmd);
    }

    public String readLine() throws IOException {
	String l = _in.readLine();
	if (debug)
	    System.out.println("SP<< " + l);
	return l;
    }

    public Matcher expect(String regexp) {
	String resp = null;

	Pattern p = Pattern.compile(regexp);
	long startMsec = System.currentTimeMillis();
	while (System.currentTimeMillis() < startMsec + TIMEOUT) {
	    try {
		resp = readLine();
	    } catch (IOException e) {}

	    if (resp != null) {
		Matcher m = p.matcher(resp);
		if (m.matches())
		    return m;
	    }
	}

	if (resp == null)
	    throw new RuntimeException("No reply from serial device within " + TIMEOUT + " ms");
	else
	    throw new UnexpectedResponseException(resp, regexp);
    }

    public void flushInput() {
	try {
	    while (_in.ready())
	        _in.read();
	} catch (IOException e) {
	    throw new RuntimeException(e);
	}
    }

    public void close() {
	_port.close(); 
    }
}
