import java.io.*;
import java.util.*;
import java.util.regex.*;

public class V60 implements CellPhone {
    private SerialPort _port;

    private static void delay(int msec) {
	try {
	    Thread.sleep(msec);
	} catch (InterruptedException e) {}
    }

    public V60(File device) {
	_port = new SerialPort(device);

	_port.println("ATZ");
	_port.println("AT+MODE=0");
	_port.println("ATZ");
	delay(500);
	_port.flushInput();

	try {
	    _port.println("ATZ");
	    _port.expect("(?:OK|ERROR)");
	} catch (UnexpectedResponseException e) {}
	_port.flushInput();

	_port.println("AT+MODE?");
	Matcher m = _port.expect(".MODE: (\\d+)");
	_port.expect("OK");
	if (!m.group(1).equals("2")) {
	    _port.println("AT+MODE=2");
	    _port.expect("OK");
	    _port.expect(".MBAN.*");
	}
    }

    public void close() {
	_port.close();
    }

    public String getPhoneNumber() {
	_port.println("AT+CNUM?");
	Matcher m = _port.expect(".CNUM: ,(\\d+),");
	_port.expect("OK");
	return m.group(1);
    }

    private Collection<PhoneBookEntry> getPhoneBookEntries(int start, int end) {
	if (end < start)
	    throw new RuntimeException("end index " + end + " is less than start index " + start);
	_port.println("AT+MPBR=" + start + "," + end);
	Collection<PhoneBookEntry> ents = new HashSet<PhoneBookEntry>();

	while (true) {
	    Matcher m = _port.expect("(?:OK|.MPBR: (\\d+),\"([^\"]+)\",\\d+,\"([^\"]+)\",(\\d+),.*)");
	    if (m.group(0).startsWith("OK"))
		return ents;

	    int idx = Integer.parseInt(m.group(1));
	    String phoneNumber = m.group(2);
	    String name = m.group(3);
	    PhoneType type = PhoneType.findByIndex(Integer.parseInt(m.group(4)));
	    PhoneBookEntry pbe = new PhoneBookEntry(idx, phoneNumber, name, type);
	    ents.add(pbe);
	}
    }

    public Collection<PhoneBookEntry> getPhoneBookEntries() {
	_port.println("AT+MPBR=?");
	Matcher m = _port.expect(".MPBR: (\\d+)-(\\d+),.*");
	_port.expect("OK");

	int startIdx = Integer.parseInt(m.group(1));
	int endIdx = Integer.parseInt(m.group(2));
	System.out.println("Phone book from " + startIdx + " to " + endIdx);

	Collection<PhoneBookEntry> ents = new HashSet<PhoneBookEntry>();
	int step = 50;
	for (int i = startIdx; i < endIdx; i += step) {
	    int iEnd = i + step - 1;
	    if (iEnd > endIdx)
		iEnd = endIdx;
	    Collection<PhoneBookEntry> stepEnts =
		getPhoneBookEntries(i, iEnd);
	    ents.addAll(stepEnts);
	}

	return ents;
    }

    public void removePhoneBookEntry(PhoneBookEntry e) {
	_port.println("AT+MPBW=" + e.index());
	_port.expect("OK");
    }

    private void setPhoneBookEntry(PhoneBookEntry e) {
	_port.println("AT+MPBW=" + e.index() + ",\"" + e.phoneNumber() +
		      "\",129,\"" + e.name() + "\"," + e.type().index() + ",0");
	_port.expect("OK");	      
    }

    public void addPhoneBookEntry(PhoneBookEntry e) {
	int idx = e.index();
	if (getPhoneBookEntries(idx, idx).size() != 0)
	    throw new RuntimeException("Phone book entry with index " + idx + " already exists");
	setPhoneBookEntry(e);
    }

    public void updatePhoneBookEntry(PhoneBookEntry e) {
	int idx = e.index();
	if (getPhoneBookEntries(idx, idx).size() == 0)
	    throw new RuntimeException("Updating a non-existant entry " + idx);
	setPhoneBookEntry(e); 
    }
}
